#include <GL/glu.h>
#include <math.h>
#include "camera.h"

void Camera::normalize() {
   posx = (posx <= 50) ? ((posx >= -50) ? posx : -50) : 50;
   posz = (posz <= 50) ? ((posz >= -50) ? posz : -50) : 50;
   posy = (posy <= 30) ? ((posy >= 0) ? posy : 0) : 30;
   ax = (ax + 360) % 360;
   ay = (ay + 360) % 360;
}

Camera::Camera() {
   posx = 0.0; posy = 3.0; posz = 0.0;
   ay = -15; ax = 0;
   view = PER;
}
      
void Camera::setPos(GLdouble x, GLdouble y, GLdouble z) {
   if(view != STATIC) {
      posx = x;
      posy = y;
      posz = z;
   }
}

void Camera::setAngles(int x, int y) {
   if(view != STATIC) {
      ax = x;
      ay = y;
   }
}

void Camera::addAngleY(GLint y) {
   if(view == PER) ay += y;
}

void Camera::addAngleX(GLint x) {
   if(view != STATIC) ax += x;
}

void Camera::goUp() {
   if(view != STATIC) {
      float a = (float)ax/180*3.142;
      posx += sin(a);
      posz -= cos(a);
   }
}

void Camera::goDown() {
   if(view != STATIC) {
      float a = (float)ax/180*3.142;
      posx -= sin(a);
      posz += cos(a);
   }
}

void Camera::addPosY(GLdouble y) {
   if(view == PER) posy += y;
}
      
char *Camera::changeView() {
   if(view == ORTO) {
      view = PER;
      look();
      return "Perspective view";
   }
   else if(view == PER) {
      view = STATIC;
      look();
      return "Static view";
   }
   else if(view == STATIC) {
      view = ORTO;
      look();
      return "Orthogonal view";
   }
}
      
void Camera::look(void) {
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if(view == ORTO) {
      glOrtho(-6.0, 6.0, -6.0, 6.0, -10, 20);
   }
   else {
      gluPerspective(60, 1, 0.5, 400);
   }
}
      
void Camera::selectLook(void) {
   if(view == ORTO) {
      glOrtho(-6.0, 6.0, -6.0, 6.0, -10, 20);
   }
   else {
      gluPerspective(60, 1, 0.5, 400);
   }
}
      
void Camera::project(void) {
   normalize();
   glLoadIdentity();
   if(view == PER) {
      glRotated(ay,1,0,0);
      glRotated(ax,0,1,0);
      glTranslated(-posx, -posy, -posz);
   }
   else if(view == ORTO) {
      glRotated(90,1,0,0);
      glRotated(ax,0,1,0);
      glTranslated(-posx, -5, -posz);
   }
   else {
      glRotated(15,1,0,0);
      glRotated(-45,0,1,0);
      glTranslated(-70, -40, -70);
   }
}
