#include <GL/gl.h>
#include <GL/glut.h>
#include "engine.h"
#include "spruce.h"
#include "leaftree.h"
#include "bench.h"

Engine::Engine() {
   mode = MOVEX;
   sunmove = true;
   skymove = true;
   lighting = true;
   fog = true;

   cam = new Camera();
   text = new TextViewer();
   sph = new Sphere();
   sun = new Sun();
   desk = new Surface();
   store = new Storage("data/data.txt");

   this->change();
}

Engine::~Engine() {
   delete cam;
   delete text;
   delete sph;
   delete sun;
   delete store;
}

void Engine::change(void) {
   if(lighting) glEnable(GL_LIGHTING);
   else glDisable(GL_LIGHTING);

   if(fog) glEnable(GL_FOG);
   else glDisable(GL_FOG);
}

void Engine::draw(void) {
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glMatrixMode(GL_MODELVIEW);
   cam->project();
   sun->draw(sunmove);
   desk->draw();   
   sph->draw(skymove);
   store->show();
   text->show();
   glutSwapBuffers();
}

void Engine::drag(int x, int y) {
   switch(mode) {
      case MOVEX:
         store->moveX(sign(x - lastx));
         break;
      case MOVEY:
         store->moveY(sign(x - lastx));
         break;
      case ROT:
         store->rotate(5*sign(x - lastx));
         break;
   }
   lastx = x;
}

int Engine::sign(int x) {
   if(x < 0) return -1;
   if(x > 0) return 1;
   return 0;
}

void Engine::menu(int item) {
   Object *obj;
   switch(item) {
      case 0:
         exit(0);
         break;
      case 1:
         text->add(cam->changeView());
         break;
      case 2:
         sunmove = !sunmove;
         if(sunmove) text->add("Sun is moving");
         else text->add("Sun stops");
         break;
      case 3:
         skymove = !skymove;
         if(skymove) text->add("Clouds are moving");
         else text->add("Clouds stops");
         break;
      case 4:
         lighting = !lighting;
         if(lighting) text->add("Lights are on");
         else text->add("Lights are off");
         change();
         break;
      case 5:
         fog = !fog;
         if(fog) text->add("Fog is on");
         else text->add("Fog is off");
         change();
         break;
      case 6:
         glutFullScreen();
         text->add("Going to fullscreen. No way back...");
         break;
      case 7:
         store->save("data.txt");
         text->add("Scene saved");
         break;
      case 100:
         obj = new Spruce(0, 0, 0);
         store->add(obj);
         text->add(obj->getName());
         break;
      case 101:
         obj = new LeafTree(0, 0, 0);
         store->add(obj);
         text->add(obj->getName());
         break;
      case 102:
         obj = new Bench(0, 0, 0);
         store->add(obj);
         text->add(obj->getName());
         break;
      case 200:
         if(store->isSelected()) {
            store->erase();
            text->add("Deleted...");
         }
         else {
            text->add("Nothing selected");
         }
         break;
      case 201:
         mode = MOVEX;
         break;
      case 202:
         mode = MOVEY;
         break;
      case 203:
         mode = ROT;
         break;
      case 204:
         store->deselect();
         text->add("Clear selection");
         break;
   }
}

void Engine::key(unsigned char key, int x, int y) {
	switch (key) {
		case 27 : 
         store->deselect();
         text->add("Clear selection");
			break;
		case 'c':
         text->add(cam->changeView());
         break; 
		case 'w':
         cam->addPosY(1);
         break; 
		case 's':
         cam->addPosY(-1);
         break; 
   }
}

void Engine::speckey(int key, int x, int y) {
	switch (key) {
		case GLUT_KEY_UP:
         cam->goUp();
         break;
		case GLUT_KEY_DOWN:
         cam->goDown();
         break;
		case GLUT_KEY_LEFT:
         cam->addAngleX(-1);
         break;
		case GLUT_KEY_RIGHT:
         cam->addAngleX(1);
         break;
		case GLUT_KEY_PAGE_UP:
         cam->addAngleY(-1);
         break;
		case GLUT_KEY_PAGE_DOWN:
         cam->addAngleY(1);
         break;
   }
}

void Engine::mouse(int button, int state, int x, int y) {
   switch(button) {
      case GLUT_LEFT_BUTTON:
         if(state == GLUT_DOWN) {
            if(!store->isSelected()) {
               text->add(store->select(x, y, cam));
            }
            lastx = x;
         }
         break;
      case GLUT_MIDDLE_BUTTON:
         if(mode == MOVEX) mode = MOVEY;
         else if(mode == MOVEY) mode = ROT;
         else if(mode == ROT) mode = MOVEX;
         break;
      case GLUT_RIGHT_BUTTON:
         break;
   }
}

void Engine::resize(int width, int height) {
   this->width = width;
   this->height = height;
   glViewport(0, 0, width, height);
   cam->look();
}
