#include <stdio.h>
#include <string.h>
#include <Gl/glu.h>
#include "font.h"

#define CHAR_WIDTH 22
#define CHAR_HEIGHT 24
#define REAL_CHAR_WIDTH 10.0
#define NUM_CHARS 256
#define TAB_CHARS 4.0

Font::Font() {
   fontTexture = new Texture("data/font.tga");
   texture = fontTexture->getTexID();
   base = glGenLists(NUM_CHARS);

   int loop;
   float cx, cy;
   
   for(loop = 0; loop < NUM_CHARS; loop++) {
      glNewList(base + loop, GL_COMPILE);
      if(loop < 33) {
         if(loop == '\t') glTranslated(REAL_CHAR_WIDTH * TAB_CHARS, 0.0, 0.0);
         else glTranslated(REAL_CHAR_WIDTH, 0.0, 0.0);
      }
      else {
         cx = (float)(loop % 16) / 16.0f;
         cy = (float)(loop / 16) / 16.0f;
              
         glBegin(GL_QUADS);
           glTexCoord2f(cx, 1.0f - cy - 0.0625f); glVertex2i(0, 0);
           glTexCoord2f(cx + 0.0625f, 1.0f - cy - 0.0625f); glVertex2i(CHAR_WIDTH, 0);
           glTexCoord2f(cx + 0.0625f, 1.0f - cy); glVertex2i(CHAR_WIDTH, CHAR_HEIGHT);
           glTexCoord2f(cx, 1.0f - cy); glVertex2i(0, CHAR_HEIGHT);
         glEnd();
         glTranslated(REAL_CHAR_WIDTH, 0.0, 0.0);// Presun na pravou stranu znaku
      }
      glEndList();
   }
}

Font::~Font() {
   glDeleteLists(base, NUM_CHARS);
   delete fontTexture;
}

void Font::begin() {
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   gluOrtho2D(0, 800, 0, 600);
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity();
             
   glPushAttrib(GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT | GL_POLYGON_BIT | GL_LIST_BIT);

   glDisable(GL_LIGHTING);
   glDisable(GL_DEPTH_TEST);
   glEnable(GL_CULL_FACE);
   glEnable(GL_TEXTURE_2D);
   glEnable(GL_BLEND);

   glPolygonMode(GL_FRONT, GL_FILL);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glBindTexture(GL_TEXTURE_2D, texture);
   glListBase(base + 96);
}
        
void Font::end() {
   glPopAttrib();
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();
}

void Font::printf(int x, int y, const char* text, ...) {
   if(text == NULL) return;
             
   char string[256];
   va_list ap;
   va_start(ap, text);
   vsprintf(string, text, ap);
   va_end(ap);
             
   glLoadIdentity();
   glTranslated(x, y, 0.0);
   glCallLists(strlen(string), GL_UNSIGNED_BYTE, string);
}
