#include <stdio.h>
#include <stdlib.h>
#include <GL/glu.h>
#include "storage.h"
#include "spruce.h"
#include "leaftree.h"
#include "bench.h"

Storage::Storage(char *filename) {
   head = NULL;
   selected = 0;

   int typ;
   char str[255];
   FILE *file = fopen(filename, "rt");
   if(file != NULL) {
      while(!feof(file)) {
         fscanf(file, " ");
         typ = fgetc(file);
         if((typ == '#') || (typ == '\r') || (typ == '\n')) {
            fgets(str, 255, file);
            continue;
         }
         int x,y;
         switch(typ) {
            case 'S':
               fscanf(file, " %d %d", &x, &y);
               if(head == NULL) {
                  head = new Spruce(x, y, 0);
               }
               else {
                  head->setLast(new Spruce(x, y, 0));
               }
               printf("Smrk: %d, %d\n", x, y);
               break;
            case 'K':
               fscanf(file, " %d %d", &x, &y);
               if(head == NULL) {
                  head = new LeafTree(x, y, 0);
               }
               else {
                  head->setLast(new LeafTree(x, y, 0));
               }
               printf("Listnac: %d, %d\n", x, y);
               break;
            case 'L':
               int rot;
               fscanf(file, " %d %d %d", &x, &y, &rot);
               if(head == NULL) {
                  head = new Bench(x, y, rot);
               }
               else {
                  head->setLast(new Bench(x, y, rot));
               }
               printf("Lavicka: %d, %d; %d\n", x, y, rot);
               break;
/*            default:
               printf("Error in input file: %c", typ);
               getchar();
               exit(1);*/
         }
      }
      fclose(file);
   }
}

Storage::~Storage() {
   Object *pom = head;
   while(head) {
      head = head->getNext();
      delete pom;
      pom = head;
   }
}

void Storage::show(void) {
   Object *pom = head;
   while(pom) {
      pom->draw(false);
      if(pom->getIndex() == selected) pom->drawBounds();
      pom = pom->getNext();
   }
}

#define MAXSELECT 512

char *Storage::select(int x, int y, Camera *cam) {
   GLuint selectBuf[MAXSELECT];
   GLint hits;
   GLint vp[4];

   glSelectBuffer(MAXSELECT, selectBuf);
   (void)glRenderMode(GL_SELECT);
   glInitNames();
   glPushName(0);
   
   glPushMatrix();
   glGetIntegerv(GL_VIEWPORT, vp);

   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   gluPickMatrix(x, vp[3] - y, 4, 4, vp);
   cam->selectLook();
   glMatrixMode(GL_MODELVIEW);

   glClear(GL_COLOR_BUFFER_BIT);

   cam->project();
   Object *pom = head;
   while(pom) {
      pom->draw(true);
      pom = pom->getNext();
   }

   glPopMatrix();
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glFlush();
   glMatrixMode(GL_MODELVIEW);
    
   hits = glRenderMode(GL_RENDER); 

   GLuint *ptr = selectBuf, sel = 0, names;
   float max = -1;
   for (int i = 0; i < hits; i++) {
      names = *ptr; ptr++;
      float z1 = *ptr; ptr++;
      float z2 = *ptr; ptr++;
      if((z1 > max) || (z2 > max)) sel = *ptr;
      ptr++;
      for (int j = 1; j < names; j++) { // tohle by nemelo probehnout...
         printf("%d ", *ptr); ptr++;
      }
   }
   
   selected = sel;
   pom = head;  
   while(pom) {
      if(pom->getIndex() == sel) {
         return pom->getName();
      }
      pom = pom->getNext();
   }
   return "nothing...";
}

void Storage::add(Object *object) {
   if(head == NULL) {
      head = object;
   }
   else {
      head->setLast(object);
   }
   selected = object->getIndex();
}

void Storage::erase(void) {
   if(selected) {
      Object *pom1 = head;
      if(head) {
         if(pom1->getIndex() == selected) {
            selected = 0;
            head = head->getNext();
            delete pom1;
            return;
         }
         Object *pom2 = head;
         pom1 = head->getNext();
         while(pom1) {
            if(pom1->getIndex() == selected) {
               selected = 0;
               Object *p = pom1;
               pom2->setNext(pom1->getNext());
               delete pom1;
               return;
            }
            pom2 = pom1;
            pom1 = pom1->getNext();
         }
      }
   }
}

void Storage::moveX(int dx) {
   if(selected) {
      Object *pom = head;
      while(pom) {
         if(pom->getIndex() == selected) {
            pom->moveX(dx);
         }
         pom = pom->getNext();
      }
   }
}

void Storage::moveY(int dy) {
   if(selected) {
      Object *pom = head;
      while(pom) {
         if(pom->getIndex() == selected) {
            pom->moveY(dy);
         }
         pom = pom->getNext();
      }
   }
}

void Storage::rotate(int angle) {
   if(selected) {
      Object *pom = head;
      while(pom) {
         if(pom->getIndex() == selected) {
            pom->rotate(angle);
         }
         pom = pom->getNext();
      }
   }
}

void Storage::save(char * filename) {
   FILE *file = fopen(filename, "wt");
   fputs("# generated output", file);
   Object *pom = head;
   while(pom) {
      fputs(pom->save(), file);
      pom = pom->getNext();
   }
   fclose(file);
}
