#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <GL/glu.h>
#include "sun.h"

#define RADIUS 100.0
/*
GLfloat shineSpecular[] = {0.628281, 0.555802, 0.366065, 1.0};
GLfloat shineDiffuse[] = {0.75164, 0.60648, 0.22648, 1.0};
GLfloat shineAmbient[] = {0.24725, 0.1995, 0.0745, 1.0};
GLfloat shineShininess[] = {51.2};
GLfloat shineEmission[] = {0.1, 0.1, 0.1, 1.0};
*/
GLfloat shineSpecular[] = {0.5, 0.5, 0.5, 0.5};
GLfloat shineDiffuse[] = {0.5, 0.5, 0.5, 0.5};
GLfloat shineAmbient[] = {0.5, 0.5, 0.5, 0.5};
GLfloat shineShininess[] = {51.2};
GLfloat shineEmission[] = {0.3, 0.3, 0.3, 1.0};

Sun::Sun() {
   ax = 0.03;
   float a = (float)ax/180*3.142;
   cosa = cos(a); sina = sin(a);
   sun_position[0] = 0.0; sun_position[1] = -RADIUS; sun_position[2] = 0.0; sun_position[3] = 0.0;
   sun_ambient[0] = sun_ambient[1] = sun_ambient[2] = 0.0; sun_ambient[3] = 1.0;
   sun_difuse[0] = sun_difuse[1] = sun_difuse[2] = sun_difuse[3] = 1.0;
   
   glLightfv(GL_LIGHT0, GL_AMBIENT, sun_ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, sun_difuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR, sun_difuse);
   glEnable(GL_LIGHT0);
   glFogi(GL_FOG_MODE, GL_EXP2);
   glFogf(GL_FOG_DENSITY, 0.0);

   int sun = glGenLists(1);
   if(sun != 0) {
      glNewList(sun, GL_COMPILE);
        glBindTexture(GL_TEXTURE_2D, 0);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, shineSpecular);
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, shineDiffuse);
        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, shineAmbient);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shineShininess);

        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, shineEmission);
        GLUquadricObj *quadratic = gluNewQuadric();
        gluQuadricTexture(quadratic, GL_FALSE);
        gluQuadricOrientation(quadratic, GLU_INSIDE);
        gluQuadricNormals(quadratic, GLU_SMOOTH);

        gluSphere(quadratic, 1, 8, 8);
        gluDeleteQuadric(quadratic);
      glEndList();
   }
   else {
      printf("Cannot create list.");
      getchar();
      exit(1);
   }
}

Sun::~Sun() {
}

void Sun::draw(bool move) {
   static bool sunrise = false;
   static GLfloat fog_density = 0.0;
   if(move) {
      float x = cosa * sun_position[0] - sina * sun_position[1];
      float y = sina * sun_position[0] + cosa * sun_position[1];
      if(y > 1) {
         for(int i = 0; i < 7; i++) {
            sun_position[0] = x;
            sun_position[1] = y;
            x = (cosa * sun_position[0]) - (sina * sun_position[1]);
            y = (sina * sun_position[0]) + (cosa * sun_position[1]);
         }
      }
      sun_position[0] = x;
      sun_position[1] = y;
   }

   glLightfv(GL_LIGHT0, GL_DIFFUSE, sun_difuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR, sun_difuse);
   glLightfv(GL_LIGHT0, GL_POSITION, sun_position);
   glPushMatrix();
     glTranslated(sun_position[0], sun_position[1], sun_position[2]);
     glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, shineSpecular);
     glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, shineDiffuse);
     glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, shineAmbient);
     glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shineShininess);
     glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, shineEmission);
     glCallList(sun);
   glPopMatrix();

   if(sun_position[1] <= -1) {
     sunrise = false;
     fog_density = 0.0;
     glFogf(GL_FOG_DENSITY, fog_density);
   }
   else {
      if(sunrise && (sun_difuse[0] <= 1)) {
         sun_difuse[0] = sun_difuse[1] = sun_difuse[2] = sun_difuse[3] += 0.00001;
      }
      else if(!sunrise && sun_difuse[0] >= 0.1) {
         sun_difuse[0] = sun_difuse[1] = sun_difuse[2] = sun_difuse[3] -= 0.00001;
      }
      if(sunrise && (fog_density >= 0.0)) {
         fog_density -= 0.0002;
      }
      else if(!sunrise && fog_density < 1) {
         fog_density += 0.0002;
      }
      glFogf(GL_FOG_DENSITY, fog_density);
   }
   if(sun_position[1] > RADIUS - 1) {
      sunrise = true;
   }
}
