#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "texture.h"

bool Texture::LoadTGA(char *filename) {
   GLubyte TGAheader[12] = {0,0,2,0,0,0,0,0,0,0,0,0};
   GLubyte TGAcompare[12];
   GLubyte header[6];
   GLuint bytesPerPixel;
   GLuint imageSize;
   GLuint type = GL_RGBA;

   FILE *file = fopen(filename, "rb");
   if((file == NULL) || (fread(TGAcompare, 1, sizeof(TGAcompare), file) != sizeof(TGAcompare)) || (memcmp(TGAheader, TGAcompare, sizeof(TGAheader)) != 0) || (fread(header, 1, sizeof(header), file) != sizeof(header))) {
      if(file == NULL) return false;
      else {
         fclose(file);
         return false;
      }
   }

   width  = header[1] * 256 + header[0];
   height = header[3] * 256 + header[2];
   if((width <= 0) || (height <= 0) || ((header[4] != 24) && (header[4] != 32))) {
      fclose(file);
      return false;
   }

   bpp = header[4];
   bytesPerPixel = bpp/8;
   imageSize = width * height * bytesPerPixel;
   data = new GLubyte[imageSize];

   if((data == NULL) || (fread(data, 1, imageSize, file) != imageSize)) {
      if(data != NULL) free(data);
      fclose(file);
      return false;
   }

   for(GLuint i = 0; i < int(imageSize); i += bytesPerPixel)	{
      GLuint temp = data[i];
      data[i] = data[i + 2];
      data[i + 2] = temp;
   }

   fclose(file);

   texID = 0;
   glGenTextures(1, &texID);
   glBindTexture(GL_TEXTURE_2D, texID);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

   if(bpp == 24) type = GL_RGB;

   glTexImage2D(GL_TEXTURE_2D, 0, type, width, height, 0, type, GL_UNSIGNED_BYTE, data);

   return true;
}

Texture::Texture(char *filename) {
   if(!LoadTGA(filename)) {
      printf("Can not load texture in %s. Exiting...\n", filename);
      getchar();
      exit(1);
   }
}

Texture::~Texture() {
   delete data;
}
